exports.__esModule = true;
exports['default'] = getWebpackErrors;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _libFns = require('../../lib/fns');

var lineSep = '\n  ';
var takeWebpack = function takeWebpack(ls) {
  return lineSep + _libFns._.take(ls, 2).join(lineSep);
};
var split = function split(s) {
  return s.split("\n");
};
var join = function join(s) {
  return s.join("\n");
};

function cleanPath(str) {
  return str.trim().replace(/\.\/\.flint(\/\.internal)?(\/deps(\/internal)?)?/g, '').replace(new RegExp(_opts2['default']('appDir'), 'g'), '').replace(new RegExp('Module not found: ', 'g'), '').replace('in /' + _libFns.path.relative(_opts2['default']('appDir'), _opts2['default']('deps').dir), '') // webpack remove "in /deps/externals.in.js"
  .replace("/externals.in.js\n", '').replace("/internals.in.js\n", '');
}

function getWebpackErrors(where, err, stats) {
  if (err) return err;

  var jsonStats = stats.toJson({
    source: false
  });

  // debug
  if (_opts2['default']('debug')) {
    _libFns.log.webpack('--- webpack output ---');
    _libFns.log.webpack(jsonStats.modules.map(function (s) {
      return ('' + s.name)[s.built && !s.failed ? 'green' : 'red'];
    }).join("\n"));
  }

  // check errors
  var errors = jsonStats.errors;

  if (errors.length) {
    // debug output everything
    _libFns.log.webpack('webpackErrors', errors);

    var messages = errors.map(split).map(takeWebpack)[0].dim;
    var whereMsg = where == 'externals' ? 'NPM modules' : 'imported local modules';
    var message = cleanPath(('Webpack: ' + whereMsg).yellow + ('  ' + messages)).grey;

    var file = undefined,
        line = undefined,
        column = undefined;

    // this is ridiculous, but webpack only gives us errors as a big string stack dump
    // with the file at the bottom and line number at the end
    try {
      var lastError = errors[errors.length - 1].split("\n");
      var fileAndLine = cleanPath(lastError[lastError.length - 1]).split(' '); // on last line of webpack error

      if (fileAndLine.length > 2) {
        file = fileAndLine[1];
        line = fileAndLine[2].split(':')[0];
        column = fileAndLine[2].split(':')[1];
      }
    } catch (e) {
      return e;
    }

    // rather than error out and break startup of app from running,
    // we just log directly but continue
    return { message: message, file: file, loc: { line: line, column: column } };
  }

  // check warnings
  if (jsonStats.warnings.length) {
    print('Webpack warnings: ', jsonStats.warnings[0].split("\n").slice(0, 3).join("\n"));
  }

  _libFns.log.webpack('webpack finished');
}

module.exports = exports['default'];