exports.__esModule = true;
exports['default'] = webpackConfig;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _libFns = require('../../lib/fns');

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _cache = require('../../cache');

var _cache2 = _interopRequireDefault(_cache);

var _flintJs = require('flint-js');

var _flintJs2 = _interopRequireDefault(_flintJs);

var runnerRoot = _libFns.path.resolve(_libFns.path.join(__dirname, '..', '..', '..', '..'));
var runnerModules = _libFns.path.join(runnerRoot, 'node_modules');

// copy styles into .flint/static/styles
var fileStyleFolder = 'file?name=assets/styles/[name]-[hash].css';

function webpackConfig(filename) {
  var config = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var conf = _deepmerge2['default']({
    context: runnerRoot,
    debug: _opts2['default']('debug'),
    output: {
      // app/.flint/.internal/deps
      path: _opts2['default']('deps').dir,
      filename: filename,
      library: _opts2['default']('saneName') + '-' + filename.replace('.js', ''),
      libraryTarget: 'commonjs'
    },
    // come from flint.js
    externals: {
      react: 'exports.React',
      'react-dom': 'exports.ReactDOM',
      history: 'exports.history',
      radium: 'exports.radium'
    },
    devtool: 'source-map',
    node: {
      global: false,
      Buffer: false,
      setImmediate: false
    },
    resolveLoader: { root: runnerModules },
    resolve: {
      root: _opts2['default']('flintDir'),
      modulesDirectories: ['node_modules'],
      extensions: ['', '.js', '.jsx', '.css'],
      fallback: [runnerModules]
    },

    // path.join(flintRoot, '..') // fallback to app root for import css
    module: {
      loaders: [{ test: /\.(ttf|eot|woff|svg)$/, loader: 'file?name=assets/files/[name]-[hash].[ext]' }, { test: /\.json$/, loader: 'json' }, {
        test: /\.css$/,
        loaders: [fileStyleFolder, 'css?module&importLoaders=1']
      }, {
        test: /\.(png|jpg|gif)$/,
        loader: 'url?limit=8192&name=[name]-[hash].[ext]'
      }]
    }
  }, config);

  _libFns.log('bundler', '------------webpackConfig------------'.yellow, conf);

  return conf;
}

module.exports = exports['default'];