exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _stream = require('stream');

var _vinyl = require('vinyl');

var _vinyl2 = _interopRequireDefault(_vinyl);

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _helpers = require('./helpers');

var _index = require('../index');

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _cache = require('../../cache');

var _cache2 = _interopRequireDefault(_cache);

var _bridge = require('../../bridge');

var _bridge2 = _interopRequireDefault(_bridge);

var _libFns = require('../../lib/fns');

// time we wait for browser load before we just force push
var UPPER_WAIT_LIMIT = 2000;

var isFileType = function isFileType(_path, ext) {
  return _libFns.path.extname(_path) == '.' + ext;
};
var debug = _libFns.log.bind(null, { name: 'stream', icon: '🎏' });

var basePath = undefined,
    flintPath = undefined,
    relPath = undefined;
var internalTimeout = undefined;
var browserLoading = {};
var queue = {};
var stream = new _stream.Readable({ objectMode: true });
stream._read = function (n) {};

function init() {
  basePath = _opts2['default']('appDir');
  flintPath = _opts2['default']('flintDir');
  relPath = function (p) {
    return _path3['default'].relative(basePath, p);
  };
  watchForBrowserLoading();

  // watch, throttle the stream a bit
  _bridge2['default'].onMessage('live:save', _libFns._.throttle(fileSend, 22, { leading: true }));

  // reset loading on errors in pipeline
  _index.event('error', function (_ref) {
    var path = _ref.path;
    return setBrowserLoading(relPath(path), false);
  });
}

// ignore stream when loading file in browser
function watchForBrowserLoading() {
  _bridge2['default'].onMessage('script:load', function (_ref2) {
    var path = _ref2.path;
    return setBrowserLoading(path, true);
  });
  _bridge2['default'].onMessage('script:done', function (_ref3) {
    var path = _ref3.path;
    return setBrowserLoading(path, false);
  });
}

function setBrowserLoading(path, isLoading) {
  browserLoading[path] = isLoading;
  debug('IN', 'browser', isLoading ? 'loading'.red : 'done'.green, path);
  if (!isLoading) loadWaiting(path);
}

function fileSend(_ref4) {
  var path = _ref4.path;
  var startTime = _ref4.startTime;
  var contents = _ref4.contents;

  // check if file actually in flint project
  if (!path || path.indexOf(basePath) !== 0 || relPath(path).indexOf('.flint') === 0 || !isFileType(path, 'js')) {
    debug('  file not js || not in path || in .flint', path);
    return;
  }

  // write to stream
  var relative = relPath(path);
  var sendImmediate = _cache2['default'].isInternal(path);

  debug('SIN', relative);

  pushStreamRun(relative, function () {
    debug('SOUT', relative);
    queue[relative] = false;
    // we may get another stream in before browser even starts loading
    // setBrowserLoading(relative, true)
    var file = new _vinyl2['default'](_libFns.vinyl(basePath, path, new Buffer(contents)));

    var stackTime = [{
      name: 'fileSend',
      time: +Date.now() - startTime
    }];
    Object.assign(file, { startTime: startTime, stackTime: stackTime });

    stream.push(file);
  }, sendImmediate);
}

function pushStreamRun(relative, finish, sendImmediate) {
  // waiting for script load
  if (!browserLoading[relative] || sendImmediate) return finish();

  // only queue once
  if (queue[relative]) return;

  queue[relative] = finish;

  // ensure upper limit on wait
  setTimeout(function () {
    if (!queue[relative]) return;

    debug('upper limit! finish'.yellow);
    browserLoading[relative] = false;
    finish();
  }, UPPER_WAIT_LIMIT);
}

// load waiting
function loadWaiting(path) {
  var queued = queue[relPath(path)];
  if (queued) queued();
}

exports['default'] = {
  init: init,
  stream: stream
};
module.exports = exports['default'];