exports.__esModule = true;

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, 'next'); var callThrow = step.bind(null, 'throw'); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _fns = require('./fns');

// uses a simple lock to ensure reads and writes are done safely

exports['default'] = _asyncToGenerator(function* (filePath, _ref) {

  // public

  var read = _asyncToGenerator(function* () {
    try {
      if (cache) return cache;
      var state = yield _fns.readFile(filePath);

      cacheStr = state;

      if (json) {
        state = JSON.parse(state);
        cache = state;
      }

      return state;
    } catch (e) {
      logw('read error'.red, e);
      return defaultValue;
    }
  });

  var write = _asyncToGenerator(function* (writer) {
    try {
      // logw('waiting...')
      if (lock) yield lock;
      var _unlock = getLock();
      var state = yield read();
      var result = yield stateWriter(writer, state, _unlock);
    } catch (e) {
      _fns.handleError(e);
    }
  });

  var _ref$debug = _ref.debug;
  var debug = _ref$debug === undefined ? '' : _ref$debug;
  var _ref$json = _ref.json;
  var json = _ref$json === undefined ? false : _ref$json;
  var _ref$defaultValue = _ref.defaultValue;
  var defaultValue = _ref$defaultValue === undefined ? '' : _ref$defaultValue;

  // helpers
  var logw = _fns.log.writer.bind(null, debug.yellow);

  // constructor
  try {
    yield read();
  } catch (e) {
    try {
      yield write(function (_, write) {
        return write(defaultValue);
      });
    } catch (e) {
      _fns.handleError(e);
    }
  }

  // internal state
  var cache = null;
  var cacheStr = null;
  var isChanged = true;

  function hasChanged() {
    var result = isChanged;
    logw('hasChanged?', result);
    isChanged = false;
    return result;
  }

  // private
  var lock = null;
  var unlock = null;

  function getLock() {
    // logw('lock', lock)

    if (lock) return unlock;else {
      // logw('no lock, returning new')
      lock = new Promise(function (res) {
        unlock = function () {
          lock = null;
          res();
        };
      });

      return unlock;
    }
  }

  function stateWriter(writer, state, unlock) {
    // logw('about to call writer...')
    return new Promise(function (res, rej) {
      writer(state, _asyncToGenerator(function* (toWrite) {
        var finish = function finish() {
          unlock();
          res();
        };

        var toWriteRaw = undefined;

        // do this before doing equality checks for cache
        if (json) {
          toWriteRaw = toWrite;
          toWrite = JSON.stringify(toWrite, null, 2);
        }

        if (cacheStr == toWrite) {
          logw('hasnt changed!'.bold);
          finish();
          return;
        }

        try {
          yield _fns.writeFile(filePath, toWrite);

          isChanged = true;
          cacheStr = toWrite;
          cache = toWriteRaw;

          finish();
        } catch (e) {
          rej(e);
        }
      }));
    });
  }

  // API
  return {
    read: read,
    write: write,
    hasChanged: hasChanged
  };
});
module.exports = exports['default'];