exports.__esModule = true;
exports['default'] = log;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _opts = require('../opts');

var _opts2 = _interopRequireDefault(_opts);

var debug = false;

var subIcons = {
  IN: ' ⇠ '.blue,
  OUT: ' ⇢ '.bold.green,
  SIN: ' ⇜ '.magenta,
  SOUT: ' ⇝ '.bold.yellow,
  RELOAD: ' ↺ '.yellow,
  STAR: ' ★ '.yellow,
  PIPE: ' | '.gray,
  CHECK: ' ✓ ',
  X: ' ✖ ',
  PLUS: ' ✚ ',
  ASTERISK: ' ✺ ',
  HAPPY: ' ☺ ',
  SAD: ' ☹ ',
  WRITE: ' ✍ ',
  DOWN: ' ↓ ',
  UP: ' ↑ '
};

function log(info, subIcon) {
  for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  if (!debug) return;

  var isInfo = typeof info == 'object' && info.name && info.icon;

  var icon = undefined,
      subiout = undefined;

  if (isInfo) {
    var subi = subIcons[subIcon];
    subiout = subi ? subi : '    ' + subIcon;
    args = [info.icon, subiout].concat(args);
  } else args = [info, subIcon].concat(args);

  var doLog = function doLog() {
    return print.apply(undefined, ['   '].concat(colorArgs(args)));
  };

  // all
  if (!debug.length) return doLog();
  // filtered
  if (info.name && debug.indexOf(info.name) >= 0) doLog();
}

log.externals = log.bind(null, { name: 'externals', icon: '🚀' });
log.internals = log.bind(null, { name: 'internals', icon: '🏠' });
log.cache = log.bind(null, { name: 'cache', icon: '💰' });
log.opts = log.bind(null, { name: 'opts', icon: '❍' });
log.gulp = log.bind(null, { name: 'gulp', icon: '👇' });
log.writer = log.bind(null, { name: 'writer', icon: '✎' });
log.file = log.bind(null, { name: 'file', icon: '▻' });
log.webpack = log.bind(null, { name: 'webpack', icon: '🎒' });

function colorArgs(args) {
  return args.map(function (arg) {
    return typeof arg === 'boolean' ? !!arg ? ('' + arg).bold.green : ('' + arg).bold.red : arg;
  });
}

log.setLogging = function () {
  debug = _opts2['default']('debug');
};

var timeName = null;
var startTime = null;

log.start = function (name) {
  timeName = name;
  startTime = Date.now();
};

log.end = function () {
  print('TIME', timeName, Date.now() - startTime);
};
module.exports = exports['default'];